# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Base class for Spark Job."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse

from apitools.base.py import encoding

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.dataproc.jobs import base as job_base
from googlecloudsdk.command_lib.dataproc.jobs import util as job_util


class SparkBase(job_base.JobBase):
  """Submit a Java or Scala Spark job to a cluster."""

  @staticmethod
  def Args(parser):
    """Parses command-line arguments specific to submitting Spark jobs."""
    parser.add_argument(
        '--jars',
        type=arg_parsers.ArgList(),
        metavar='JAR',
        default=[],
        help=('Comma separated list of jar files to be provided to the '
              'executor and driver classpaths.'))
    parser.add_argument(
        '--files',
        type=arg_parsers.ArgList(),
        metavar='FILE',
        default=[],
        help=('Comma separated list of files to be placed in the working '
              'directory of both the app master and executors.'))
    parser.add_argument(
        '--archives',
        type=arg_parsers.ArgList(),
        metavar='ARCHIVE',
        default=[],
        help=(
            'Comma separated list of archives to be extracted into the working '
            'directory of each executor. '
            'Must be one of the following file formats: .zip, .tar, .tar.gz, '
            'or .tgz.'))
    parser.add_argument(
        'job_args',
        nargs=argparse.REMAINDER,
        help='Arguments to pass to the driver.')
    parser.add_argument(
        '--properties',
        type=arg_parsers.ArgDict(),
        metavar='PROPERTY=VALUE',
        help='List of key value pairs to configure Spark. For a list of '
             'available properties, see: '
             'https://spark.apache.org/docs/latest/'
             'configuration.html#available-properties.')
    parser.add_argument(
        '--properties-file',
        help=job_util.PROPERTIES_FILE_HELP_TEXT)
    parser.add_argument(
        '--driver-log-levels',
        type=arg_parsers.ArgDict(),
        metavar='PACKAGE=LEVEL',
        help=('List of package to log4j log level pairs to configure driver '
              'logging. For example: root=FATAL,com.example=INFO'))

  @staticmethod
  def GetFilesByType(args):
    """Returns a dict of files by their type (jars, archives, etc.)."""
    return {
        'main_jar': args.main_jar,
        'jars': args.jars,
        'archives': args.archives,
        'files': args.files}

  @staticmethod
  def ConfigureJob(messages, job, files_by_type, logging_config, args):
    """Populates the sparkJob member of the given job."""

    spark_job = messages.SparkJob(
        args=args.job_args or [],
        archiveUris=files_by_type['archives'],
        fileUris=files_by_type['files'],
        jarFileUris=files_by_type['jars'],
        mainClass=args.main_class,
        mainJarFileUri=files_by_type['main_jar'],
        loggingConfig=logging_config)

    job_properties = job_util.BuildJobProperties(
        args.properties, args.properties_file)
    if job_properties:
    # Sort properties to ensure tests comparing messages not fail on ordering.
      spark_job.properties = encoding.DictToAdditionalPropertyMessage(
          job_properties, messages.SparkJob.PropertiesValue, sort_items=True)

    job.sparkJob = spark_job



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common arguments for `gcloud source repos` commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.source import resource_args
from googlecloudsdk.command_lib.util.concepts import concept_parsers

REPO_NAME_VALIDATOR = arg_parsers.RegexpValidator(
    '[A-Za-z0-9_][-._A-Za-z0-9/]{0,127}',
    'repostory name may contain between 1 and 128 (inclusive) letters, digits, '
    'hyphens, periods, underscores and slashes.')


def AddPushblockFlags(group):
  """Add pushblock enabled/disabled flags to the given group."""

  group.add_argument(
      '--enable-pushblock',
      action='store_true',
      help="""\
Enable PushBlock for all repositories under current project.
PushBlock allows repository owners to block git push transactions containing
private key data.""")

  group.add_argument(
      '--disable-pushblock',
      action='store_true',
      help="""\
Disable PushBlock for all repositories under current project.
PushBlock allows repository owners to block git push transactions containing
private key data.""")


def AddOptionalTopicFlags(group, resource_name='repo'):
  """Add message_format and service_account flags to the topic arg group."""
  group.add_argument(
      '--message-format',
      choices=['json', 'protobuf'],
      help="""\
The format of the message to publish to the topic.""")

  group.add_argument(
      '--service-account',
      help="""\
Email address of the service account used for publishing Cloud Pub/Sub messages.
This service account needs to be in the same project as the {}. When added, the
caller needs to have iam.serviceAccounts.actAs permission on this service
account. If unspecified, it defaults to the Compute Engine default service
account.""".format(resource_name))

  group.add_argument(
      '--topic-project',
      help="""\
Cloud project for the topic. If not set, the currently set project will be
used.""")


def AddRepoUpdateArgs(parser, verb='to update'):
  """Add the arg groups for `source repos update` command."""
  topic_group = parser.add_group(
      'Manages Cloud Pub/Sub topics associated with the repository.',
      required=True)
  topic_resource_group = topic_group.add_argument_group(
      mutex=True, required=True)

  concept_parsers.ConceptParser(
      [
          resource_args.CreateTopicResourcePresentationSpec(
              'add', 'The Cloud Pub/Sub topic to add to the repository.',
              topic_resource_group),
          resource_args.CreateTopicResourcePresentationSpec(
              'remove',
              'The Cloud Pub/Sub topic to remove from the repository.',
              topic_resource_group),
          resource_args.CreateTopicResourcePresentationSpec(
              'update', 'The Cloud Pub/Sub topic to update in the project.',
              topic_resource_group),
          resource_args.CreateRepoResourcePresentationSpec(verb)
      ],
      command_level_fallthroughs={
          '--add-topic.project': ['--topic-project'],
          '--remove-topic.project': ['--topic-project'],
          '--update-topic.project': ['--topic-project'],
      }).AddToParser(parser)

  AddOptionalTopicFlags(topic_group)


def AddProjectConfigUpdateArgs(parser):
  """Add the arg groups for `source project-configs update` command."""
  update_group = parser.add_group(required=True, mutex=True)
  AddPushblockFlags(update_group)
  topic_group = update_group.add_group(
      'Manage Cloud Pub/Sub topics associated with the Cloud project.')
  topic_resource_group = topic_group.add_argument_group(mutex=True)

  concept_parsers.ConceptParser(
      [
          resource_args.CreateTopicResourcePresentationSpec(
              'add', 'The Cloud Pub/Sub topic to add to the project.',
              topic_resource_group),
          resource_args.CreateTopicResourcePresentationSpec(
              'remove', 'The Cloud Pub/Sub topic to remove from the project.',
              topic_resource_group),
          resource_args.CreateTopicResourcePresentationSpec(
              'update', 'The Cloud Pub/Sub topic to update in the project.',
              topic_resource_group),
      ],
      command_level_fallthroughs={
          '--add-topic.project': ['--topic-project'],
          '--remove-topic.project': ['--topic-project'],
          '--update-topic.project': ['--topic-project'],
      }).AddToParser(parser)
  AddOptionalTopicFlags(topic_group, 'project')



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for modifying the target of forwarding rules."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import forwarding_rules_utils as utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.forwarding_rules import flags


class SetTargetHelper(object):
  """Helper that sets a forwarding rule's target."""

  FORWARDING_RULE_ARG = None

  def __init__(self, holder, include_regional_tcp_proxy):
    self._holder = holder
    self._include_regional_tcp_proxy = include_regional_tcp_proxy

  @classmethod
  def Args(cls, parser, include_regional_tcp_proxy):
    """Adds flags to set the target of a forwarding rule."""
    cls.FORWARDING_RULE_ARG = flags.ForwardingRuleArgument()
    flags.AddSetTargetArgs(
        parser,
        include_regional_tcp_proxy=include_regional_tcp_proxy)
    cls.FORWARDING_RULE_ARG.AddArgument(parser)

  def Run(self, args):
    """Issues requests necessary to set target on Forwarding Rule."""
    client = self._holder.client

    forwarding_rule_ref = self.FORWARDING_RULE_ARG.ResolveAsResource(
        args,
        self._holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client))

    if forwarding_rule_ref.Collection() == 'compute.globalForwardingRules':
      requests = self.CreateGlobalRequests(client, self._holder.resources,
                                           forwarding_rule_ref, args)
    elif forwarding_rule_ref.Collection() == 'compute.forwardingRules':
      requests = self.CreateRegionalRequests(client, self._holder.resources,
                                             forwarding_rule_ref, args)

    return client.MakeRequests(requests)

  def CreateGlobalRequests(self, client, resources, forwarding_rule_ref, args):
    """Create a globally scoped request."""
    target_ref = utils.GetGlobalTarget(resources, args)

    request = client.messages.ComputeGlobalForwardingRulesSetTargetRequest(
        forwardingRule=forwarding_rule_ref.Name(),
        project=forwarding_rule_ref.project,
        targetReference=client.messages.TargetReference(
            target=target_ref.SelfLink(),),
    )

    return [(client.apitools_client.globalForwardingRules, 'SetTarget', request)
           ]

  def CreateRegionalRequests(self, client, resources, forwarding_rule_ref,
                             args):
    """Create a regionally scoped request."""
    target_ref, _ = utils.GetRegionalTarget(
        client,
        resources,
        args,
        forwarding_rule_ref=forwarding_rule_ref,
        include_regional_tcp_proxy=self._include_regional_tcp_proxy)

    request = client.messages.ComputeForwardingRulesSetTargetRequest(
        forwardingRule=forwarding_rule_ref.Name(),
        project=forwarding_rule_ref.project,
        region=forwarding_rule_ref.region,
        targetReference=client.messages.TargetReference(
            target=target_ref.SelfLink(),),
    )

    return [(client.apitools_client.forwardingRules, 'SetTarget', request)]


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Set(base.UpdateCommand):
  """Modify a forwarding rule to direct network traffic to a new target."""

  FORWARDING_RULE_ARG = None
  _include_regional_tcp_proxy = False

  detailed_help = {
      'DESCRIPTION': ("""
          *{{command}}* is used to set a new target for a forwarding
          rule. {overview}

          When creating a forwarding rule, exactly one of  ``--target-instance'',
          ``--target-pool'', ``--target-http-proxy'', ``--target-https-proxy'',
          ``--target-grpc-proxy'', ``--target-ssl-proxy'',
          ``--target-tcp-proxy'' or ``--target-vpn-gateway''
          must be specified.""".format(overview=flags.FORWARDING_RULES_OVERVIEW)
                     ),
  }

  @classmethod
  def Args(cls, parser):
    SetTargetHelper.Args(parser, cls._include_regional_tcp_proxy)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    return SetTargetHelper(holder, self._include_regional_tcp_proxy).Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class SetBeta(Set):
  """Modify a forwarding rule to direct network traffic to a new target."""
  _include_regional_tcp_proxy = True


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class SetAlpha(SetBeta):
  """Modify a forwarding rule to direct network traffic to a new target."""
  _include_regional_tcp_proxy = True



"""Generated client library for policyanalyzer version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.policyanalyzer.v1 import policyanalyzer_v1_messages as messages


class PolicyanalyzerV1(base_api.BaseApiClient):
  """Generated client library for service policyanalyzer version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://policyanalyzer.googleapis.com/'
  MTLS_BASE_URL = 'https://policyanalyzer.mtls.googleapis.com/'

  _PACKAGE = 'policyanalyzer'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'PolicyanalyzerV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new policyanalyzer handle."""
    url = url or self.BASE_URL
    super(PolicyanalyzerV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_activityTypes_activities = self.ProjectsLocationsActivityTypesActivitiesService(self)
    self.projects_locations_activityTypes = self.ProjectsLocationsActivityTypesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsActivityTypesActivitiesService(base_api.BaseApiService):
    """Service class for the projects_locations_activityTypes_activities resource."""

    _NAME = 'projects_locations_activityTypes_activities'

    def __init__(self, client):
      super(PolicyanalyzerV1.ProjectsLocationsActivityTypesActivitiesService, self).__init__(client)
      self._upload_configs = {
          }

    def Query(self, request, global_params=None):
      r"""Queries policy activities on Google Cloud resources.

      Args:
        request: (PolicyanalyzerProjectsLocationsActivityTypesActivitiesQueryRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudPolicyanalyzerV1QueryActivityResponse) The response message.
      """
      config = self.GetMethodConfig('Query')
      return self._RunMethod(
          config, request, global_params=global_params)

    Query.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/activityTypes/{activityTypesId}/activities:query',
        http_method='GET',
        method_id='policyanalyzer.projects.locations.activityTypes.activities.query',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/activities:query',
        request_field='',
        request_type_name='PolicyanalyzerProjectsLocationsActivityTypesActivitiesQueryRequest',
        response_type_name='GoogleCloudPolicyanalyzerV1QueryActivityResponse',
        supports_download=False,
    )

  class ProjectsLocationsActivityTypesService(base_api.BaseApiService):
    """Service class for the projects_locations_activityTypes resource."""

    _NAME = 'projects_locations_activityTypes'

    def __init__(self, client):
      super(PolicyanalyzerV1.ProjectsLocationsActivityTypesService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(PolicyanalyzerV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(PolicyanalyzerV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for Database Migration Service commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def ConnectionProfileAttributeConfig(name='connection_profile'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The connection profile of the {resource}.')


def MigrationJobAttributeConfig(name='migration_job'):
  return concepts.ResourceParameterAttributeConfig(
      name=name, help_text='The migration job of the {resource}.')


def RegionAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='region', help_text='The Cloud region for the {resource}.')


def GetRegionResourceSpec(resource_name='region'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations',
      resource_name=resource_name,
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetConnectionProfileResourceSpec(resource_name='connection_profile'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.connectionProfiles',
      resource_name=resource_name,
      connectionProfilesId=ConnectionProfileAttributeConfig(name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def GetMigrationJobResourceSpec(resource_name='migration_job'):
  return concepts.ResourceSpec(
      'datamigration.projects.locations.migrationJobs',
      resource_name=resource_name,
      migrationJobsId=MigrationJobAttributeConfig(name=resource_name),
      locationsId=RegionAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def AddConnectionProfileResourceArg(parser, verb, positional=True):
  """Add a resource argument for a database migration connection profile.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    positional: bool, if True, means that the resource is a positional rather
      than a flag.
  """
  if positional:
    name = 'connection_profile'
  else:
    name = '--connection-profile'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetConnectionProfileResourceSpec(),
      'The connection profile {}.'.format(verb),
      required=True).AddToParser(parser)


def AddCloudSqlConnectionProfileResouceArgs(parser, verb):
  """Add resource arguments for a database migration CloudSQL connection profile.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'connection_profile',
          GetConnectionProfileResourceSpec(),
          'The connection profile {}.'.format(verb),
          required=True),
      presentation_specs.ResourcePresentationSpec(
          '--source-id',
          GetConnectionProfileResourceSpec(),
          'Database Migration Service source connection profile ID.',
          required=True,
          flag_name_overrides={'region': ''}),
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source-id.region': ['--region']
      }).AddToParser(parser)


def AddMigrationJobResourceArgs(parser, verb, required=False):
  """Add resource arguments for creating/updating a database migration job.

  Args:
    parser: argparse.ArgumentParser, the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    required: boolean, whether source/dest resource args are required.
  """
  resource_specs = [
      presentation_specs.ResourcePresentationSpec(
          'migration_job',
          GetMigrationJobResourceSpec(),
          'The migration job {}.'.format(verb),
          required=True),
      presentation_specs.ResourcePresentationSpec(
          '--source',
          GetConnectionProfileResourceSpec(),
          'ID of the source connection profile, representing the source database.',
          required=required,
          flag_name_overrides={'region': ''}),
      presentation_specs.ResourcePresentationSpec(
          '--destination',
          GetConnectionProfileResourceSpec(),
          'ID of the destination connection profile, representing the '
          'destination database.',
          required=required,
          flag_name_overrides={'region': ''})
  ]
  concept_parsers.ConceptParser(
      resource_specs,
      command_level_fallthroughs={
          '--source.region': ['--region'],
          '--destination.region': ['--region']
      }).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource arguments for GCE Image Packages."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def ImagesAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='image',
      help_text='Name of the image.')


def GetImageResourceSpec():
  return concepts.ResourceSpec(
      'compute.images',
      resource_name='image',
      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False)


def CreateImageResourcePresentationSpec(group_help, image_prefix=''):
  name, flag_name_overrides = '--image', {}
  if image_prefix:
    name = '--{}-image'.format(image_prefix)
    flag_name_overrides = {'project': '--{}-project'.format(image_prefix)}
  return presentation_specs.ResourcePresentationSpec(
      name,
      GetImageResourceSpec(),
      group_help=group_help,
      required=True,
      prefixes=False,
      flag_name_overrides=flag_name_overrides,
  )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for Fleet memberships commands."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.container.fleet.memberships import errors


def SetInitProjectPath(ref, args, request):
  """Set the appropriate request path in project attribute for initializeHub requests.

  Args:
    ref: reference to the membership object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """
  del ref, args  # Unused.
  request.project = request.project + '/locations/global/memberships'
  return request


def SetParentCollection(ref, args, request):
  """Set parent collection with location for created resources.

  Args:
    ref: reference to the membership object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """
  del ref, args  # Unused.
  request.parent = request.parent + '/locations/-'
  return request


def GetConnectGatewayServiceName(endpoint_override, location):
  """Get the appropriate Connect Gateway endpoint.

  This function checks the environment endpoint overide configuration for
  Fleet and uses it to determine which Connect Gateway endpoint to use.
  The overridden Fleet value will look like
  https://autopush-gkehub.sandbox.googleapis.com/.

  When there is no override set, this command will return a Connect Gateway
  prod endpoint. When an override is set, an appropriate non-prod endpoint
  will be provided instead.

  For example, when the overridden value looks like
  https://autopush-gkehub.sandbox.googleapis.com/,
  the function will return
  autopush-connectgateway.sandbox.googleapis.com.

  Regional prefixes are supported via the location argument. For example, when
  the overridden value looks like
  https://autopush-gkehub.sandbox.googleapis.com/ and location is passed in as
  "us-west1", the function will return
  us-west1-autopush-connectgateway.sandbox.googleapis.com.

  Args:
    endpoint_override: The URL set as the API endpoint override for 'gkehub'.
      Empty string if the override is not set.
    location: The location against which the command is supposed to run. This
      will be used to dynamically modify the service name to a location-specific
      value. If this is the value 'global' or None, a global service name is
      returned.

  Returns:
    The service name to use for this command invocation, optionally modified
    to target a specific location.

  Raises:
    UnknownApiEndpointOverrideError if the Fleet API endpoint override is not
    one of the standard values.
  """

  # Determine the location prefix, if any
  prefix = '' if location in ('global', None) else '{}-'.format(location)
  if not endpoint_override:
    # endpoint_override will be empty string for Prod.
    return '{}connectgateway.googleapis.com'.format(prefix)
  elif 'autopush-gkehub' in endpoint_override:
    return '{}autopush-connectgateway.sandbox.googleapis.com'.format(prefix)
  elif 'staging-gkehub' in endpoint_override:
    return '{}staging-connectgateway.sandbox.googleapis.com'.format(prefix)
  else:
    raise errors.UnknownApiEndpointOverrideError('gkehub')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Patches for pickling isssues in apitools.

The storage surface needs to be able to serialize apitools messages to support
multiprocessing; however, there are a number of bugs with pickling apitools
messages that need to be patched, pending more permanent fixes.
"""

# TODO(b/171296237): Remove this file when fixes are submitted in apitools.

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.protorpclite import messages


def _time_zone_offset_init_args(self):
  """Implements apitools.base.protorpclite.util.TimeZoneOffset.__getinitargs__.

  The apitools TimeZoneOffset class inherits from datetime.datetime, which
  implements custom picking behavior in __reduce__. This reduce method cannot
  handle the additional argument that TimeZoneOffset adds to __init__, which
  makes TimeZoneOffset unpicklable without implementing __getinitargs__ as
  we do here.

  Args:
    self (TimeZoneOffset): an instance of TimeZoneOffset.

  Returns:
    A tuple of arguments passed to TimeZoneOffset.__init__ when unpickling.
  """
  # pylint: disable=protected-access
  return (self._TimeZoneOffset__offset,)
  # pylint: enable=protected-access


def patch():
  messages.util.TimeZoneOffset.__getinitargs__ = _time_zone_offset_init_args



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library containing resource args used by Transcoder commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


def LocationAttributeConfig():
  fallthroughs = [
      deps.PropertyFallthrough(properties.VALUES.transcoder.location)
  ]
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='Transcoder location for resources',
      fallthroughs=fallthroughs)


def TemplateAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='template_id', help_text='Transcoder template id for job')


def JobAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='job_name', help_text='Transcoder job name')


def GetLocationResourceSpec():
  return concepts.ResourceSpec(
      'transcoder.projects.locations',
      resource_name='location',
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG)


def GetJobResourceSpec():
  """Constructs and returns the Resource specification for Job."""

  return concepts.ResourceSpec(
      'transcoder.projects.locations.jobs',
      resource_name='job',
      jobsId=JobAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      disable_auto_completers=False)


def GetTemplateResourceSpec():
  """Constructs and returns the Resource specification for Job Template."""

  return concepts.ResourceSpec(
      'transcoder.projects.locations.jobTemplates',
      resource_name='jobTemplate',
      jobTemplatesId=TemplateAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      disable_auto_completers=False)


def AddLocationResourceArg(parser):
  """Constructs and returns the Location Resource Argument."""
  return concept_parsers.ConceptParser.ForResource(
      '--location',
      GetLocationResourceSpec(),
      'Transcoder location',
      required=True).AddToParser(parser)


def AddJobResourceArg(parser):
  """Constructs and returns the Job Resource Argument."""

  return concept_parsers.ConceptParser.ForResource(
      'job_name',
      GetJobResourceSpec(),
      'Transcoder Job name',
      required=True).AddToParser(parser)


def AddTemplateResourceArg(parser):
  """Constructs and returns Job Template Resource Argument."""

  return concept_parsers.ConceptParser.ForResource(
      'template_id',
      GetTemplateResourceSpec(),
      'Transcoder job template id',
      required=True).AddToParser(parser)



"""Generated client library for file version v1beta1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.file.v1beta1 import file_v1beta1_messages as messages


class FileV1beta1(base_api.BaseApiClient):
  """Generated client library for service file version v1beta1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://file.googleapis.com/'
  MTLS_BASE_URL = 'https://file.mtls.googleapis.com/'

  _PACKAGE = 'file'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'FileV1beta1'
  _URL_VERSION = 'v1beta1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new file handle."""
    url = url or self.BASE_URL
    super(FileV1beta1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_backups = self.ProjectsLocationsBackupsService(self)
    self.projects_locations_instances_shares = self.ProjectsLocationsInstancesSharesService(self)
    self.projects_locations_instances_snapshots = self.ProjectsLocationsInstancesSnapshotsService(self)
    self.projects_locations_instances = self.ProjectsLocationsInstancesService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsBackupsService(base_api.BaseApiService):
    """Service class for the projects_locations_backups resource."""

    _NAME = 'projects_locations_backups'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsBackupsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a backup.

      Args:
        request: (FileProjectsLocationsBackupsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/backups',
        http_method='POST',
        method_id='file.projects.locations.backups.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['backupId'],
        relative_path='v1beta1/{+parent}/backups',
        request_field='backup',
        request_type_name='FileProjectsLocationsBackupsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a backup.

      Args:
        request: (FileProjectsLocationsBackupsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/backups/{backupsId}',
        http_method='DELETE',
        method_id='file.projects.locations.backups.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsBackupsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific backup.

      Args:
        request: (FileProjectsLocationsBackupsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Backup) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/backups/{backupsId}',
        http_method='GET',
        method_id='file.projects.locations.backups.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsBackupsGetRequest',
        response_type_name='Backup',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all backups in a project for either a specified location or for all locations.

      Args:
        request: (FileProjectsLocationsBackupsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListBackupsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/backups',
        http_method='GET',
        method_id='file.projects.locations.backups.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+parent}/backups',
        request_field='',
        request_type_name='FileProjectsLocationsBackupsListRequest',
        response_type_name='ListBackupsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the settings of a specific backup.

      Args:
        request: (FileProjectsLocationsBackupsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/backups/{backupsId}',
        http_method='PATCH',
        method_id='file.projects.locations.backups.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta1/{+name}',
        request_field='backup',
        request_type_name='FileProjectsLocationsBackupsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsInstancesSharesService(base_api.BaseApiService):
    """Service class for the projects_locations_instances_shares resource."""

    _NAME = 'projects_locations_instances_shares'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsInstancesSharesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a share.

      Args:
        request: (FileProjectsLocationsInstancesSharesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/shares',
        http_method='POST',
        method_id='file.projects.locations.instances.shares.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['shareId'],
        relative_path='v1beta1/{+parent}/shares',
        request_field='share',
        request_type_name='FileProjectsLocationsInstancesSharesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a share.

      Args:
        request: (FileProjectsLocationsInstancesSharesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/shares/{sharesId}',
        http_method='DELETE',
        method_id='file.projects.locations.instances.shares.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSharesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific share.

      Args:
        request: (FileProjectsLocationsInstancesSharesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Share) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/shares/{sharesId}',
        http_method='GET',
        method_id='file.projects.locations.instances.shares.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSharesGetRequest',
        response_type_name='Share',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all shares for a specified instance.

      Args:
        request: (FileProjectsLocationsInstancesSharesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSharesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/shares',
        http_method='GET',
        method_id='file.projects.locations.instances.shares.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+parent}/shares',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSharesListRequest',
        response_type_name='ListSharesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the settings of a specific share.

      Args:
        request: (FileProjectsLocationsInstancesSharesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/shares/{sharesId}',
        http_method='PATCH',
        method_id='file.projects.locations.instances.shares.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta1/{+name}',
        request_field='share',
        request_type_name='FileProjectsLocationsInstancesSharesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsInstancesSnapshotsService(base_api.BaseApiService):
    """Service class for the projects_locations_instances_snapshots resource."""

    _NAME = 'projects_locations_instances_snapshots'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsInstancesSnapshotsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a snapshot.

      Args:
        request: (FileProjectsLocationsInstancesSnapshotsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/snapshots',
        http_method='POST',
        method_id='file.projects.locations.instances.snapshots.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['snapshotId'],
        relative_path='v1beta1/{+parent}/snapshots',
        request_field='snapshot',
        request_type_name='FileProjectsLocationsInstancesSnapshotsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a snapshot.

      Args:
        request: (FileProjectsLocationsInstancesSnapshotsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/snapshots/{snapshotsId}',
        http_method='DELETE',
        method_id='file.projects.locations.instances.snapshots.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSnapshotsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific snapshot.

      Args:
        request: (FileProjectsLocationsInstancesSnapshotsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Snapshot) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/snapshots/{snapshotsId}',
        http_method='GET',
        method_id='file.projects.locations.instances.snapshots.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSnapshotsGetRequest',
        response_type_name='Snapshot',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all snapshots in a project for either a specified location or for all locations.

      Args:
        request: (FileProjectsLocationsInstancesSnapshotsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListSnapshotsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/snapshots',
        http_method='GET',
        method_id='file.projects.locations.instances.snapshots.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+parent}/snapshots',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesSnapshotsListRequest',
        response_type_name='ListSnapshotsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the settings of a specific snapshot.

      Args:
        request: (FileProjectsLocationsInstancesSnapshotsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}/snapshots/{snapshotsId}',
        http_method='PATCH',
        method_id='file.projects.locations.instances.snapshots.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta1/{+name}',
        request_field='snapshot',
        request_type_name='FileProjectsLocationsInstancesSnapshotsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsInstancesService(base_api.BaseApiService):
    """Service class for the projects_locations_instances resource."""

    _NAME = 'projects_locations_instances'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsInstancesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates an instance. When creating from a backup, the capacity of the new instance needs to be equal to or larger than the capacity of the backup (and also equal to or larger than the minimum capacity of the tier).

      Args:
        request: (FileProjectsLocationsInstancesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances',
        http_method='POST',
        method_id='file.projects.locations.instances.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['instanceId'],
        relative_path='v1beta1/{+parent}/instances',
        request_field='instance',
        request_type_name='FileProjectsLocationsInstancesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes an instance.

      Args:
        request: (FileProjectsLocationsInstancesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='DELETE',
        method_id='file.projects.locations.instances.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force'],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a specific instance.

      Args:
        request: (FileProjectsLocationsInstancesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Instance) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='GET',
        method_id='file.projects.locations.instances.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesGetRequest',
        response_type_name='Instance',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists all instances in a project for either a specified location or for all locations.

      Args:
        request: (FileProjectsLocationsInstancesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListInstancesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances',
        http_method='GET',
        method_id='file.projects.locations.instances.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+parent}/instances',
        request_field='',
        request_type_name='FileProjectsLocationsInstancesListRequest',
        response_type_name='ListInstancesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the settings of a specific instance.

      Args:
        request: (FileProjectsLocationsInstancesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}',
        http_method='PATCH',
        method_id='file.projects.locations.instances.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta1/{+name}',
        request_field='instance',
        request_type_name='FileProjectsLocationsInstancesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Restore(self, request, global_params=None):
      r"""Restores an existing instance's file share from a backup. The capacity of the instance needs to be equal to or larger than the capacity of the backup (and also equal to or larger than the minimum capacity of the tier).

      Args:
        request: (FileProjectsLocationsInstancesRestoreRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Restore')
      return self._RunMethod(
          config, request, global_params=global_params)

    Restore.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:restore',
        http_method='POST',
        method_id='file.projects.locations.instances.restore',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}:restore',
        request_field='restoreInstanceRequest',
        request_type_name='FileProjectsLocationsInstancesRestoreRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Revert(self, request, global_params=None):
      r"""Revert an existing instance's file system to a specified snapshot.

      Args:
        request: (FileProjectsLocationsInstancesRevertRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Revert')
      return self._RunMethod(
          config, request, global_params=global_params)

    Revert.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/instances/{instancesId}:revert',
        http_method='POST',
        method_id='file.projects.locations.instances.revert',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}:revert',
        request_field='revertInstanceRequest',
        request_type_name='FileProjectsLocationsInstancesRevertRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (FileProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='file.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='FileProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (FileProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='file.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (FileProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='file.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (FileProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='file.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+name}/operations',
        request_field='',
        request_type_name='FileProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(FileV1beta1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (FileProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='file.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta1/{+name}',
        request_field='',
        request_type_name='FileProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (FileProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='file.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'includeUnrevealedLocations', 'pageSize', 'pageToken'],
        relative_path='v1beta1/{+name}/locations',
        request_field='',
        request_type_name='FileProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(FileV1beta1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for creating trusts for domains."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core.console import console_io


def GetHandshakeSecret():
  """Prompt for user input of handshake secret with target domain."""
  unused_cred = console_io.PromptPassword(
      "Please enter handshake secret with target domain. The secret will not be stored: "
  )
  return unused_cred


def AddExtraTrustCreateArgs(unused_ref, args, request):
  """Allows for the handshake secret to be read from stdin if not specified."""
  if args.IsSpecified("handshake_secret"):
    return request
  secret = GetHandshakeSecret()
  request.attachTrustRequest.trust.trustHandshakeSecret = secret
  return request



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Artifact Registry repositories."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


class Repositories(base.Group):
  """Manage Artifact Registry tags.

  ## EXAMPLES

  To create tag with the name `my-tag` for version `1.0.0` of package `my-pkg`
  in the current project with `artifacts/repository` and `artifacts/location`
  properties are set, run:

    $ {command} create my-tag --package=my-pkg --version=1.0.0

  To list all tags under package `my-pkg`, run:

    $ {command} list --package=my-pkg

  To update tag `my-tag` from a different version to version `1.0.0` of package
  `my-pkg`, run:

    $ {command} update my-tag --version=1.0.0 --package=my-pkg

  To delete tag `my-tag` of package `my-pkg`, run:

    $ {command} delete my-tag --package=my-pkg
  """

  category = base.CI_CD_CATEGORY



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides common arguments for the Events command surface."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import collections

from googlecloudsdk.api_lib.events import trigger
from googlecloudsdk.api_lib.kuberun.core import events_constants
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.events import exceptions
from googlecloudsdk.command_lib.iam import iam_util as core_iam_util
from googlecloudsdk.command_lib.run import exceptions as run_exceptions
from googlecloudsdk.core import yaml

_DISALLOWED_SPECIAL_KEYS = [
    trigger.SOURCE_TRIGGER_LINK_FIELD, trigger.DEPENDENCY_ANNOTATION_FIELD
]


def AddSourceFlag(parser):
  """Adds a source flag. This refers to a source kind, not instance."""
  parser.add_argument(
      '--source',
      required=False,
      help='Events source kind by which to filter results.')


def AddEventTypePositionalArg(parser, release_track):
  """Adds event type positional arg."""
  if release_track == base.ReleaseTrack.ALPHA:
    help_text = 'Type of event (e.g. com.google.cloud.auditlog.event).'
  else:
    help_text = 'Type of event (e.g. google.cloud.audit.log.v1.written).'
  parser.add_argument('event_type', help=help_text)


def AddTargetServiceFlag(parser, required=False):
  """Adds target service flag."""
  parser.add_argument(
      '--target-service',
      required=required,
      help='Name or absolute uri of the Cloud Run service at which '
      'events should be received. (e.g. service-name or '
      'http://service-name.default.svc.cluster.local)')


def AddEventTypeFlagArg(parser, release_track):
  """Adds event type flag arg."""
  if release_track == base.ReleaseTrack.ALPHA:
    help_text = 'Type of event (e.g. com.google.cloud.auditlog.event).'
  else:
    help_text = (
        'Type of event (e.g. google.cloud.audit.log.v1.written). '
        'Required for all sources, except optional for ApiServerSource.')
  parser.add_argument('--type', default=None, required=False, help=help_text)


def AddBrokerFlag(parser):
  """Adds broker flag."""
  parser.add_argument(
      '--broker',
      default='default',
      help='Name of the Broker to send events to. '
      'Defaults to \'default\' if not specified.')


def AddBrokerArg(parser):
  """Adds broker arg."""
  parser.add_argument(
      'BROKER',
      help='Name of the Broker to create.')


def AddAuthenticationFlag(parser):
  """Adds authentication flag."""
  parser.add_argument(
      '--authentication',
      default='secrets',
      required=False,
      choices=events_constants.AUTH_CHOICES,
      help='Authentication mode to initialize eventing.')


def AddControlPlaneServiceAccountFlag(parser):
  """Adds service account flag."""
  parser.add_argument(
      '--service-account',
      type=core_iam_util.GetIamAccountFormatValidator(),
      help='Email address of an existing IAM service account which '
      'represents the identity of the internal events operator. If no '
      'service account is provided, a default service account ({}) will be '
      'created.'.format(events_constants.EVENTS_CONTROL_PLANE_SERVICE_ACCOUNT))


def AddBrokerServiceAccountFlag(parser):
  """Adds broker service account flag."""
  parser.add_argument(
      '--broker-service-account',
      type=core_iam_util.GetIamAccountFormatValidator(),
      help='Email address of an existing events broker IAM service account. '
      'If no service account is provided, a default service account ({}) will '
      'be created.'.format(events_constants.EVENTS_BROKER_SERVICE_ACCOUNT))


def AddSourcesServiceAccountFlag(parser):
  """Adds sources service account flag."""
  parser.add_argument(
      '--sources-service-account',
      type=core_iam_util.GetIamAccountFormatValidator(),
      help='Email address of an existing events sources IAM service account. '
      'If no service account is provided, a default service account ({}) will '
      'be created.'.format(events_constants.EVENTS_SOURCES_SERVICE_ACCOUNT))


def AddCustomEventTypeFlag(parser):
  """Adds custom event type boolean flag."""
  parser.add_argument(
      '--custom-type',
      action='store_true',
      help='If specified, the provided event type should be interpreted as a '
      'custom event type.')


def _AllowedKeyString(key):
  if key in _DISALLOWED_SPECIAL_KEYS:
    raise run_exceptions.ArgumentError('Filter {} not allowed'.format(key))
  return key


def AddFiltersFlags(parser):
  """Adds trigger filters flags."""
  parser.add_argument(
      '--trigger-filters',
      type=arg_parsers.ArgDict(key_type=_AllowedKeyString, value_type=str),
      action=arg_parsers.UpdateAction,
      metavar='KEY=VALUE',
      help="""Filters to apply before forwarding events for the target service. Events must contain all KEY=VALUE pairs to be forwarded.""",
      default=collections.OrderedDict())


_PARAMETERS_FLAG_NAME = 'parameters'


def AddParametersFlags(parser):
  """Adds parameters and parameters-from-file flags."""
  parameters_group = parser.add_mutually_exclusive_group()
  parameters_group.add_argument(
      '--{}'.format(_PARAMETERS_FLAG_NAME),
      type=arg_parsers.ArgDict(),
      action=arg_parsers.UpdateAction,
      default={},
      help='Comma-separated list of parameter names and values. '
      'Names must be one of the parameters shown when describing the '
      'event type. Only simple values can be specified with this flag. '
      'To specify more complex types like lists and nested objects, '
      'use --{}-from-file.'.format(_PARAMETERS_FLAG_NAME),
      metavar='PARAMETER=VALUE')
  parameters_group.add_argument(
      '--{}-from-file'.format(_PARAMETERS_FLAG_NAME),
      type=arg_parsers.YAMLFileContents(validator=yaml.dict_like),
      default={},
      help='Path to a local JSON or YAML file that defines a dictionary of '
      'parameters and their values. Parameters must match the items shown when '
      'describing the event type.')

_SECRETS_FLAG_NAME = 'secrets'


def AddSecretsFlag(parser):
  """Adds secrets flag."""
  parser.add_argument(
      '--{}'.format(_SECRETS_FLAG_NAME),
      type=arg_parsers.ArgDict(),
      action=arg_parsers.UpdateAction,
      default={},
      help='Comma-separated list of secret parameter names and secrets. '
      'Specify secret parameters and the secret name and key to '
      'reference. Parameter names must be one of the secret parameters shown '
      'when describing the event type.',
      metavar='PARAMETER=NAME:KEY')


_COPY_DEFAULT_SECRET = 'copy-default-secret'


def AddCopyDefaultSecret(parser):
  """Adds copy default secret flag."""
  parser.add_argument(
      '--{}'.format(_COPY_DEFAULT_SECRET),
      action='store_true',
      required=False,
      help='Copy default secret google-cloud-sources-key from cloud-run-events'
      ' namespace.')


def ValidateSourceOrTypeFlagSet(args):
  if not args.type:
    if not args.source:
      raise calliope_exceptions.OneOfArgumentsRequiredException(
          ['source', 'type'],
          'Requires at least one of either --source or --type flag.')
    elif args.source != 'ApiServerSource':
      raise calliope_exceptions.InvalidArgumentException(
          'type',
          'Source {} requires --type to be specified.'.format(args.source))


def _ParseSecretParameters(args):
  """Parses --secrets flag into dict(param -> {'name': secret name, 'key': secret key})."""
  secret_params = {}
  for param, secret_pair in getattr(args, _SECRETS_FLAG_NAME).items():
    try:
      name, key = secret_pair.split(':', 1)
      secret_params[param] = {'name': name, 'key': key}
    except ValueError:
      raise calliope_exceptions.InvalidArgumentException(
          '--{}'.format(_SECRETS_FLAG_NAME),
          'Secret name and key not property specified for [{}].'.format(
              param))
  return secret_params


_RESOURCES_FLAG_NAME = 'resources'


def AddResourcesFlag(parser):
  """Adds resources flag."""
  parser.add_argument(
      '--{}'.format(_RESOURCES_FLAG_NAME),
      action='append',
      help='Comma-separated list of secret parameter names and secrets. '
      'Specify secret parameters and the secret name and key to '
      'reference. Parameter names must be one of the secret parameters shown '
      'when describing the event type.',
      metavar='PARAMETER=NAME:KEY')


def _ParseResourcesParameters(args):
  """Parses --resources flag."""

  resources_flag = getattr(args, _RESOURCES_FLAG_NAME)

  if resources_flag is None:
    return {}

  result = []
  for api_version_kind_string in resources_flag:
    avk_selector = api_version_kind_string.split(':')
    if len(avk_selector) != 3 and len(avk_selector) != 2:
      raise run_exceptions.ArgumentError(
          'parameter flag resources expects Kind:'
          'ApiVersion:LabelName=value, notation.')
    elif len(avk_selector) == 2:
      result.append({
          'kind': avk_selector[0],
          'apiVersion': avk_selector[1],
      })
    elif len(avk_selector) == 3:
      match_labels_obj = {}
      for match_labels_string in avk_selector[2].split(','):
        key, value = match_labels_string.split('=')
        match_labels_obj[key] = value
      result.append({
          'kind': avk_selector[0],
          'apiVersion': avk_selector[1],
          'selector': {
              'matchLabels': match_labels_obj,
          }
      })

  resources_params = {}
  if result:
    resources_params['resources'] = result
  return resources_params


def _CheckUnknownParameters(event_type, known_params, given_params):
  """Raises an error if any given params are unknown."""
  unknown_parameters = (
      set(given_params) -
      set(known_params))
  if unknown_parameters:
    raise exceptions.UnknownEventTypeParameters(
        unknown_parameters, event_type)


def _CheckMissingRequiredParameters(event_type, required_params, given_params):
  """Raises an error if any required params are missing."""
  missing_parameters = (
      set(required_params) -
      set(given_params))
  if missing_parameters:
    raise exceptions.MissingRequiredEventTypeParameters(
        missing_parameters, event_type)


def _ValidateParameters(event_type, parameters, properties='properties'):
  """Validates given params conform to what's expected."""
  # TODO(b/142421197): Validate nested objects and convert non-string types
  _CheckUnknownParameters(
      event_type,
      [p.name for p in getattr(event_type.crd, properties)],
      parameters.keys())
  _CheckMissingRequiredParameters(
      event_type,
      [p.name for p in getattr(event_type.crd, properties) if p.required],
      parameters.keys())


def GetAndValidateParameters(args, event_type):
  """Validates all source parameters and returns a dict of values."""
  # Check the passed parameters for unknown keys or missing required keys
  parameters = {}
  from_file_flag = '{}_from_file'.format(_PARAMETERS_FLAG_NAME)
  if args.IsSpecified(from_file_flag):
    parameters.update(getattr(args, from_file_flag))
  if args.IsSpecified(_PARAMETERS_FLAG_NAME):
    parameters.update(getattr(args, _PARAMETERS_FLAG_NAME))

  resource_parameters = _ParseResourcesParameters(args)
  parameters.update(resource_parameters)

  _ValidateParameters(event_type, parameters)

  # Check the passed secret parameters for unknown keys or missing required keys
  secret_parameters = _ParseSecretParameters(args)
  _ValidateParameters(
      event_type, secret_parameters, properties='secret_properties')

  parameters.update(secret_parameters)
  return parameters



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared resource flags for Cloud DNS commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util import completers
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


class PolicyCompleter(completers.ListCommandCompleter):

  def __init__(self, api_version, **kwargs):
    super(PolicyCompleter, self).__init__(
        collection='dns.policies',
        api_version=api_version,
        list_command='alpha dns policies list --format=value(name)',
        parse_output=True,
        **kwargs)


def PolicyAttributeConfig(api_version):
  return concepts.ResourceParameterAttributeConfig(
      name='policy',
      completer=PolicyCompleter(api_version=api_version),
      help_text='The Cloud DNS policy name {resource}.')


def ResponsePolicyAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='response-policy',
      help_text='The Cloud DNS response policy name {resource}.')


def ResponsePolicyRuleAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='response-policy-rule',
      help_text='The Cloud DNS response policy rule name {resource}.')


def ProjectAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='project',
      help_text='The Cloud project for the {resource}.',
      fallthroughs=[deps.PropertyFallthrough(properties.VALUES.core.project)])


def GetPolicyResourceSpec(api_version):
  return concepts.ResourceSpec(
      'dns.policies',
      api_version=api_version,
      resource_name='policy',
      policy=PolicyAttributeConfig(api_version=api_version),
      project=ProjectAttributeConfig())


def GetResponsePolicyResourceSpec(api_version):
  return concepts.ResourceSpec(
      'dns.responsePolicies',
      api_version=api_version,
      resource_name='response_policy',
      responsePolicy=ResponsePolicyAttributeConfig(),
      project=ProjectAttributeConfig())


def GetResponsePolicyRuleSpec(api_version):
  return concepts.ResourceSpec(
      'dns.responsePolicyRules',
      api_version=api_version,
      resource_name='response_policy_rule',
      responsePolicy=ResponsePolicyAttributeConfig(),
      responsePolicyRule=ResponsePolicyRuleAttributeConfig(),
      project=ProjectAttributeConfig())


def AddPolicyResourceArg(parser,
                         verb,
                         api_version,
                         positional=True,
                         required=True):
  """Add a resource argument for a Cloud DNS Policy.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    api_version: str, the version of the API to use.
    positional: bool, if True, means that the policy name is a positional rather
      than a flag.
    required: bool, if True, means that the arg will be required.
  """
  if positional:
    name = 'policy'
  else:
    name = '--policy'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetPolicyResourceSpec(api_version),
      'The policy {}.'.format(verb),
      required=required).AddToParser(parser)


def AddResponsePolicyResourceArg(parser,
                                 verb,
                                 api_version,
                                 positional=True,
                                 required=True):
  """Add a resource argument for a Cloud DNS Response Policy.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    api_version: str, the version of the API to use.
    positional: bool, if True, means that the policy name is a positional rather
      than a flag.
    required: bool, if True, means that the arg will be required.
  """
  if positional:
    name = 'response_policies'
  else:
    name = '--response_policies'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetResponsePolicyResourceSpec(api_version),
      'The response policy {}.'.format(verb),
      required=required).AddToParser(parser)


def AddResponsePolicyRuleArg(parser,
                             verb,
                             api_version,
                             positional=True,
                             required=True):
  """Add a resource argument for a Cloud DNS Policy Rule.

  Args:
    parser: the parser for the command.
    verb: str, the verb to describe the resource, such as 'to update'.
    api_version: str, the version of the API to use.
    positional: bool, if True, means that the policy name is a positional rather
      than a flag.
    required: bool, if True, means that the arg will be required.
  """
  if positional:
    name = 'response_policy_rule'
  else:
    name = '--response_policy_rule'
  concept_parsers.ConceptParser.ForResource(
      name,
      GetResponsePolicyRuleSpec(api_version),
      'The response policy rule {}.'.format(verb),
      flag_name_overrides={'response-policy-rule': 'response_policy_rule'},
      required=required).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for Procurement commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers


def BillingAccountAttributeConfig(name=None, raw_help_text=None):
  if raw_help_text is not None:
    help_text = raw_help_text
  else:
    help_text = 'Cloud Billing account for the Procurement {resource}.'
  return concepts.ResourceParameterAttributeConfig(
      name=name if name is not None else 'billing-account', help_text=help_text)


def AccountAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='account', help_text='Procurement Account for the {resource}.')


def EntitlementAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='entitlement',
      help_text='Procurement Entitlement for the {resource}.')


def FreeTrialAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='free-trial', help_text='Procurement free trial for the {resource}.')


def OrderAttributeConfig(name=None, raw_help_text=None):
  if raw_help_text is not None:
    help_text = raw_help_text
  else:
    help_text = 'Procurement Order for the {resource}.'
  return concepts.ResourceParameterAttributeConfig(
      name=name if name is not None else 'order', help_text=help_text)


def OrderAllocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='order-allocation',
      help_text='Procurement Order Allocation for the {resource}.')


def OperationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='operation', help_text='Procurement Operation for the {resource}.')


def GetBillingAccountResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.billingAccounts',
      resource_name='billing-account',
      billingAccountsId=BillingAccountAttributeConfig())


def GetAccountResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.billingAccounts.accounts',
      resource_name='account',
      billingAccountsId=BillingAccountAttributeConfig(),
      accountsId=AccountAttributeConfig())


def GetEntitlementResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.projects.entitlements',
      resource_name='entitlement',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      entitlementsId=EntitlementAttributeConfig())


def GetOrderResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.billingAccounts.orders',
      resource_name='order',
      billingAccountsId=BillingAccountAttributeConfig(
          raw_help_text='Cloud Billing Account for the Procurement Order. Billing account id is required if order is not specified as full resource name.'
      ),
      ordersId=OrderAttributeConfig())


def GetOrderAllocationResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.billingAccounts.orders.orderAllocations',
      resource_name='order-allocation',
      billingAccountsId=BillingAccountAttributeConfig(
          raw_help_text='Cloud Billing Account for the Procurement Order Allocation. Billing account id is required if order allocation is not specified as full resource name.'
      ),
      ordersId=OrderAttributeConfig(
          raw_help_text='Procurement Order for the Order Allocation. Order id is required if order allocation is not specified as full resource name.'
      ),
      orderAllocationsId=OrderAllocationAttributeConfig())


def GetOrderOperationResourceSpec():
  return concepts.ResourceSpec(
      'cloudcommerceconsumerprocurement.billingAccounts.orders.operations',
      resource_name='order-operation',
      billingAccountsId=BillingAccountAttributeConfig(
          name='order-operation-billing-account'),
      ordersId=OrderAttributeConfig(name='order-operation-order'),
      operationsId=OperationAttributeConfig())


def AddBillingAccountResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      '--billing-account',
      GetBillingAccountResourceSpec(),
      description,
      required=True).AddToParser(parser)


def AddAccountResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'account', GetAccountResourceSpec(), description,
      required=True).AddToParser(parser)


def AddEntitlementResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'entitlement', GetEntitlementResourceSpec(), description,
      required=True).AddToParser(parser)


def AddOrderResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'order', GetOrderResourceSpec(), description,
      required=True).AddToParser(parser)


def AddOrderParentResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      '--order', GetOrderResourceSpec(), description,
      required=True).AddToParser(parser)


def AddOrderAllocationResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'order_allocation',
      GetOrderAllocationResourceSpec(),
      description,
      required=True).AddToParser(parser)


def AddOrderOperationResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource('--order-operation',
                                            GetOrderOperationResourceSpec(),
                                            description).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Insight."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.recommender import base
from googlecloudsdk.api_lib.recommender import flag_utils


def CreateClient(release_track):
  """Creates Client."""
  api_version = flag_utils.GetApiVersion(release_track)
  return Insight(api_version)


class Insight(base.ClientBase):
  """Base Insight client for all versions."""

  def __init__(self, api_version):
    super(Insight, self).__init__(api_version)
    self._service = self._client.projects_locations_insightTypes_insights

  def _CreateMarkRequest(self, name, state, state_metadata, etag):
    """Creates MarkRequest with the specified state."""
    # Need to do it this way to dynamically set the versioned MarkRequest
    request_name = 'MarkInsight{}Request'.format(state)
    mark_request = self._GetVersionedMessage(request_name)(etag=etag)

    if state_metadata:
      metadata = encoding.DictToAdditionalPropertyMessage(
          state_metadata,
          self._GetVersionedMessage(request_name).StateMetadataValue,
          sort_items=True)
      mark_request.stateMetadata = metadata

    # Need to do it this way to dynamically set the versioned MarkRequest
    kwargs = {
        'name':
            name,
        flag_utils.ToCamelCase(self._message_prefix + request_name):
            mark_request
    }

    # Using Project message is ok for all entities if the name is correct.
    return self._GetMessage(
        'RecommenderProjectsLocationsInsightTypesInsightsMark{}Request'.format(
            state))(**kwargs)

  def Get(self, name):
    """Gets an Insight.

    Args:
      name: str, the name of the insight being retrieved.

    Returns:
      The Insight message.
    """
    # Using Project message is ok for all entities if the name is correct.
    request = self._messages.RecommenderProjectsLocationsInsightTypesInsightsGetRequest(
        name=name)
    return self._service.Get(request)

  def List(self, parent_name, page_size, limit):
    """List Insights.

    Args:
      parent_name: str, the name of the parent.
      page_size: int, The number of items to retrieve per request.
      limit: int, The maximum number of records to yield.

    Returns:
      The Insight messages.
    """
    # Using Project message is ok for all entities if the name is correct.
    request = self._messages.RecommenderProjectsLocationsInsightTypesInsightsListRequest(
        parent=parent_name)
    return list_pager.YieldFromList(
        self._service,
        request,
        batch_size_attribute='pageSize',
        batch_size=page_size,
        limit=limit,
        field='insights')

  def MarkAccepted(self, name, state_metadata, etag):
    """Mark an insight's state as ACCEPTED.

    Args:
      name: str, the name of the insight being updated.
      state_metadata: A map of metadata for the state, provided by user or
        automations systems.
      etag: Fingerprint of the Insight. Provides optimistic locking when
        updating states.

    Returns:
      The result insights after being marked as accepted
    """
    request = self._CreateMarkRequest(name, 'Accepted', state_metadata, etag)
    return self._service.MarkAccepted(request)

  def MarkActive(self, name, etag):
    """Mark an insight's state as ACTIVE.

    Args:
      name: str, the name of the insight being updated.
      etag: Fingerprint of the Insight. Provides optimistic locking when
        updating states.

    Returns:
      The result insights after being marked as active
    """
    request = self._CreateMarkRequest(name, 'Active', None, etag)
    return self._service.MarkActive(request)

  def MarkDismissed(self, name, etag):
    """Mark an insight's state as DISMISSED.

    Args:
      name: str, the name of the insight being updated.
      etag: Fingerprint of the Insight. Provides optimistic locking when
        updating states.

    Returns:
      The result insights after being marked as dismissed
    """
    request = self._CreateMarkRequest(name, 'Dismissed', None, etag)
    return self._service.MarkDismissed(request)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Client for interaction with Gateway CRUD on API Gateway API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.api_gateway import base
from googlecloudsdk.command_lib.api_gateway import common_flags


class GatewayClient(base.BaseClient):
  """Client for gateway objects on Cloud API Gateway API."""

  def __init__(self, client=None):
    base.BaseClient.__init__(self,
                             client=client,
                             message_base='ApigatewayProjectsLocationsGateways',
                             service_name='projects_locations_gateways')
    self.DefineGet()
    self.DefineDelete()
    self.DefineList('gateways')
    self.DefineUpdate('apigatewayGateway')
    self.DefineIamPolicyFunctions()

  def Create(self, gateway_ref, api_config, display_name=None, labels=None):
    """Creates a new gateway object.

    Args:
      gateway_ref: Resource, a resource reference for the gateway
      api_config: Resource, a resource reference for the gateway
      display_name: Optional display name
      labels: Optional cloud labels

    Returns:
      Long running operation.
    """
    labels = common_flags.ProcessLabelsFlag(
        labels,
        self.messages.ApigatewayGateway.LabelsValue)

    gateway = self.messages.ApigatewayGateway(
        name=gateway_ref.RelativeName(),
        labels=labels,
        apiConfig=api_config.RelativeName(),
        displayName=display_name,
        )

    req = self.create_request(
        parent=gateway_ref.Parent().RelativeName(),
        gatewayId=gateway_ref.Name(),
        apigatewayGateway=gateway,
        )
    resp = self.service.Create(req)

    return resp



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Poller for Backup for GKE resources."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


class BackupPoller(object):
  """Backup poller for polling backup until it's terminal."""

  def __init__(self, client, messages):
    self.client = client
    self.messages = messages

  def IsNotDone(self, backup, unused_state):
    del unused_state
    return not (
        backup.state == self.messages.Backup.StateValueValuesEnum.SUCCEEDED or
        backup.state == self.messages.Backup.StateValueValuesEnum.FAILED or
        backup.state == self.messages.Backup.StateValueValuesEnum.DELETING)

  def _GetBackup(self, backup):
    req = self.messages.GkebackupProjectsLocationsBackupPlansBackupsGetRequest()
    req.name = backup
    return self.client.projects_locations_backupPlans_backups.Get(req)

  def Poll(self, backup):
    return self._GetBackup(backup)

  def GetResult(self, backup):
    return self._GetBackup(backup)


class RestorePoller(object):
  """Restore poller for polling restore until it's terminal."""

  def __init__(self, client, messages):
    self.client = client
    self.messages = messages

  def IsNotDone(self, restore, unused_state):
    del unused_state
    return not (
        restore.state == self.messages.Restore.StateValueValuesEnum.SUCCEEDED or
        restore.state == self.messages.Restore.StateValueValuesEnum.FAILED or
        restore.state == self.messages.Restore.StateValueValuesEnum.DELETING)

  def _GetRestore(self, restore):
    req = self.messages.GkebackupProjectsLocationsRestorePlansRestoresGetRequest(
    )
    req.name = restore
    return self.client.projects_locations_restorePlans_restores.Get(req)

  def Poll(self, restore):
    return self._GetRestore(restore)

  def GetResult(self, restore):
    return self._GetRestore(restore)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for Fleet Cloud Build Hybrid commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import collections
import re

from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import exceptions


class Error(exceptions.Error):
  """Class for errors raised by Cloud Build commands."""


def VerifyMembership(membership):
  """Verify the format of `membership` and check the membership exists in Fleet.

  Args:
    membership: The full membership ID.

  Raises:
    Error: if the membership specification is improper.
  """

  fleet_membership_regex = r'^projects/(?P<ProjectNum>[0-9-]+)/locations/global/memberships/(?P<Membership>[a-z0-9-]+)$'
  if re.search(fleet_membership_regex, membership) is None:
    raise Error(
        'Improper membership specification. '
        'Format should be: projects/[PROJECT_NUM]/locations/global/memberships/[MEMBERSHIP-ID]'
    )
  # We're ignoring the result of this call because we just want to verify the
  # membership exists in the fleet.
  v1beta1_client = apis.GetClientInstance('gkehub', 'v1beta1')
  v1beta1_client.projects_locations_memberships.Get(
      v1beta1_client.MESSAGES_MODULE
      .GkehubProjectsLocationsMembershipsGetRequest(name=membership))


def ParseSecuritypolicy(securitypolicy, message):
  """Convert a string representation of a security policy to an enum representation.

  Args:
    securitypolicy: string representation of the security policy
    message: message module client

  Returns:
    an enum representation of the security policy
  """

  return arg_utils.ChoiceToEnum(
      securitypolicy,
      message.CloudBuildMembershipConfig.SecurityPolicyValueValuesEnum)


def GetFeatureSpecMemberships(feature, messages):
  """Returns the feature spec for every member that has Cloud Build installed.

  Args:
    feature: Cloud Build feature
    messages: message client

  Returns:
    Cloud Build Feature Specs dictionary {"membership": membershipConfig}
  """
  spec = feature.cloudbuildFeatureSpec
  if spec is None:
    return {}
  return collections.OrderedDict(
      (k, v)
      for k, v in client.HubClient.ToPyDict(spec.membershipConfigs).items()
      if v != messages.CloudBuildMembershipConfig())


def GetFeatureStateMemberships(feature):
  """Returns the feature state for every member that has Cloud Build installed.

  Args:
    feature: Cloud Build feature

  Returns:
    Cloud Build Feature States dictionary {"membership": featureState}
  """
  state = feature.featureState
  if state is None:
    return {}
  return client.HubClient.ToPyDict(state.detailsByMembership)


def MembershipSpecPatch(messages, membership, spec):
  """Builds a Feature message for updating one CloudBuildMembershipConfig.

  Args:
    messages: The v1alpha1 Fleet messages package
    membership: The membership name to use as the key.
    spec: The CloudBuildMembershipConfig to use as the value.

  Returns:
    The messages.Feature, properly populated.
  """
  spec_map = {membership: spec}
  value = client.HubClient.ToProtoMap(
      messages.CloudBuildFeatureSpec.MembershipConfigsValue, spec_map)
  return messages.Feature(
      cloudbuildFeatureSpec=messages.CloudBuildFeatureSpec(
          membershipConfigs=value))



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper methods for exporting record-sets."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from dns import name
from dns import rdata
from dns import rdataclass
from dns import rdatatype
from dns import zone
from googlecloudsdk.api_lib.dns import svcb_stub
from googlecloudsdk.core import exceptions
from googlecloudsdk.core.resource import resource_printer


# Enable support for exporting SVCB and HTTPS records.
svcb_stub.register()


class Error(exceptions.Error):
  """Base exception for all export errors."""


class UnableToExportRecordsToFile(Error):
  """Unable to export records to specified file."""


def WriteToZoneFile(zone_file, record_sets, domain):
  """Writes the given record-sets in zone file format to the given file.

  Args:
    zone_file: file, File into which the records should be written.
    record_sets: list, ResourceRecordSets to be written out.
    domain: str, The origin domain for the zone file.
  """
  zone_contents = zone.Zone(name.from_text(domain))
  for record_set in record_sets:
    rdset = zone_contents.get_rdataset(record_set.name,
                                       record_set.type,
                                       create=True)
    for rrdata in record_set.rrdatas:
      rdset.add(rdata.from_text(rdataclass.IN,
                                rdatatype.from_text(record_set.type),
                                str(rrdata),
                                origin=zone_contents.origin),
                ttl=record_set.ttl)
  zone_contents.to_file(zone_file, relativize=False)


def WriteToYamlFile(yaml_file, record_sets):
  """Writes the given record-sets in yaml format to the given file.

  Args:
    yaml_file: file, File into which the records should be written.
    record_sets: list, ResourceRecordSets to be written out.
  """
  resource_printer.Print(record_sets, print_format='yaml', out=yaml_file)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provider-neutral tools for manipulating metadata."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import json

from googlecloudsdk.command_lib.storage import posix_util
from googlecloudsdk.command_lib.storage import user_request_args_factory
from googlecloudsdk.core.cache import function_result_cache
from googlecloudsdk.core.util import files


@function_result_cache.lru(maxsize=None)
def cached_read_json_file(file_path):
  """Convert JSON file to an in-memory dict."""
  with files.FileReader(file_path) as file_reader:
    return json.load(file_reader)


def get_label_pairs_from_file(file_path):
  """Convert JSON file to a list of label keys and values."""
  # Expected JSON file format: Dict<str: str>
  labels_dict = cached_read_json_file(file_path)
  # {'key1': 'val1', 'key2': 'val2', ...} -> [('key1', 'val1'), ...]
  return list(labels_dict.items())


def get_updated_custom_metadata(existing_custom_metadata,
                                request_config,
                                file_path=None):
  """Returns a dictionary containing new custom metadata for an object.

  Assumes that the custom metadata setter, clear flag, and a group containing
  the update and flags are in a mutually exclusive group, meaning values can be
  provided for one of these three flags/groups. The preserve POSIX flag is not a
  member of this group, meaning it can be set with any of these flags.

  Args:
    existing_custom_metadata (dict): Existing custom metadata provided by an
      API.
    request_config (request_config): May contain custom metadata fields that
      should be modified.
    file_path (str|None): If present, used for parsing POSIX data from a file on
      the system for the --preserve-posix flag. This flag's presence is
      indicated by the system_posix_data field on request_config.

  Returns:
    Optional[dict] that should be the value of the storage provider's custom
    metadata field. `None` means that existing metadata should remain unchanged.
    Empty dictionary means it should be cleared.
  """
  resource_args = request_config.resource_args
  if not resource_args:
    return

  should_parse_file_posix = request_config.system_posix_data and file_path
  if (not should_parse_file_posix and
      not resource_args.custom_metadata_to_set and
      not resource_args.custom_metadata_to_remove and
      not resource_args.custom_metadata_to_update):
    return

  posix_metadata = {}
  if should_parse_file_posix:
    posix_attributes = posix_util.get_posix_attributes_from_file(file_path)
    posix_util.update_custom_metadata_dict_with_posix_attributes(
        posix_metadata, posix_attributes)

  if resource_args.custom_metadata_to_set == user_request_args_factory.CLEAR:
    # Providing preserve POSIX and clear flags means that an object's metadata
    # should only include POSIX information.
    return posix_metadata

  # POSIX metadata overrides existing values but is overridden by fields
  # provided in update, remove, and set flags.
  if resource_args.custom_metadata_to_set:
    posix_metadata.update(resource_args.custom_metadata_to_set)
    return posix_metadata

  custom_metadata = dict(existing_custom_metadata, **posix_metadata)

  # Removes fields before updating them to avoid metadata loss.
  if resource_args.custom_metadata_to_remove:
    for key in resource_args.custom_metadata_to_remove:
      if key in custom_metadata:
        del custom_metadata[key]

  if resource_args.custom_metadata_to_update:
    custom_metadata.update(resource_args.custom_metadata_to_update)

  return custom_metadata



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for workforce pools commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


def AddParentFlags(parser, verb):
  parser.add_argument(
      '--organization',
      help='The parent organization of the workforce pool{0} to {1}.'.format(
          's' if verb == 'list' else '', verb),
      required=True)


def AddLocationFlag(parser, verb):
  parser.add_argument(
      '--location',
      help='The location of the workforce pool{0} to {1}.'.format(
          's' if verb == 'list' else '', verb),
      required=True)


def ParseLocation(args):
  if not args.IsSpecified('location'):
    return 'locations/global'
  return 'locations/{}'.format(args.location)



"""Generated client library for workflows version v1beta."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.workflows.v1beta import workflows_v1beta_messages as messages


class WorkflowsV1beta(base_api.BaseApiClient):
  """Generated client library for service workflows version v1beta."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://workflows.googleapis.com/'
  MTLS_BASE_URL = 'https://workflows.mtls.googleapis.com/'

  _PACKAGE = 'workflows'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'WorkflowsV1beta'
  _URL_VERSION = 'v1beta'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new workflows handle."""
    url = url or self.BASE_URL
    super(WorkflowsV1beta, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_workflows = self.ProjectsLocationsWorkflowsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(WorkflowsV1beta.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (WorkflowsProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='workflows.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (WorkflowsProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='workflows.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (WorkflowsProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='workflows.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/operations',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsWorkflowsService(base_api.BaseApiService):
    """Service class for the projects_locations_workflows resource."""

    _NAME = 'projects_locations_workflows'

    def __init__(self, client):
      super(WorkflowsV1beta.ProjectsLocationsWorkflowsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new workflow. If a workflow with the specified name already exists in the specified project and location, the long running operation will return ALREADY_EXISTS error.

      Args:
        request: (WorkflowsProjectsLocationsWorkflowsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/workflows',
        http_method='POST',
        method_id='workflows.projects.locations.workflows.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['workflowId'],
        relative_path='v1beta/{+parent}/workflows',
        request_field='workflow',
        request_type_name='WorkflowsProjectsLocationsWorkflowsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a workflow with the specified name. This method also cancels and deletes all running executions of the workflow.

      Args:
        request: (WorkflowsProjectsLocationsWorkflowsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}',
        http_method='DELETE',
        method_id='workflows.projects.locations.workflows.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsWorkflowsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Workflow.

      Args:
        request: (WorkflowsProjectsLocationsWorkflowsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Workflow) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}',
        http_method='GET',
        method_id='workflows.projects.locations.workflows.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsWorkflowsGetRequest',
        response_type_name='Workflow',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Workflows in a given project and location. The default order is not specified.

      Args:
        request: (WorkflowsProjectsLocationsWorkflowsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListWorkflowsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/workflows',
        http_method='GET',
        method_id='workflows.projects.locations.workflows.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/workflows',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsWorkflowsListRequest',
        response_type_name='ListWorkflowsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates an existing workflow. Running this method has no impact on already running executions of the workflow. A new revision of the workflow may be created as a result of a successful update operation. In that case, such revision will be used in new workflow executions.

      Args:
        request: (WorkflowsProjectsLocationsWorkflowsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/workflows/{workflowsId}',
        http_method='PATCH',
        method_id='workflows.projects.locations.workflows.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta/{+name}',
        request_field='workflow',
        request_type_name='WorkflowsProjectsLocationsWorkflowsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(WorkflowsV1beta.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (WorkflowsProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='workflows.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (WorkflowsProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations',
        http_method='GET',
        method_id='workflows.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/locations',
        request_field='',
        request_type_name='WorkflowsProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(WorkflowsV1beta.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }
